@@ -43,6 +43,8 @@ def host_from_json(host_data):
         provider,
         host_data["host_id"],
         host_data["name"],
+        host_data["operating_system"],
+        host_data["group"],
         host_data["ip_addrs"],
         host_data["status"],
         host_data["rawdata"],
@@ -64,6 +66,8 @@ class Host:
         provider,
         host_id,
         name,
+        operating_system,
+        group,
         ip_addrs,
         status,
         rawdata,
@@ -75,6 +79,8 @@ class Host:
         self._provider = provider
         self._host_id = host_id
         self._name = name
+        self._operating_system = operating_system
+        self._group = group
         self._ip_addrs = ip_addrs
         self._status = status
         self._username = username
@@ -87,12 +93,17 @@ class Host:
         net_str = " ".join(self._ip_addrs)
 
         out = (
-            f"{self._status} {self._host_id} {self._name} {net_str} {self._username} "
-            f"{self._password}"
+            f"{self._status} {self._operating_system} {self._host_id} {self._name} "
+            f"{net_str} {self._username} {self._password}"
         )
 
         if self._error:
-            out = "\n".join([out, "Error:", object2json(self._error)])
+            out = "\n".join(
+                [
+                    out,
+                    f"Error: Host finished with error(s): {object2json(self._error)}",
+                ]
+            )
         return out
 
     def to_json(self):
@@ -101,6 +112,8 @@ class Host:
             "provider": self._provider.name,
             "host_id": self._host_id,
             "name": self._name,
+            "operating_system": self._operating_system,
+            "group": self._group,
             "ip_addrs": self._ip_addrs,
             "status": self._status,
             "username": self._username,
@@ -114,6 +127,16 @@ class Host:
         """Get host provisioning provider."""
         return self._provider
 
+    @property
+    def operating_system(self):
+        """Get host operating system."""
+        return self._operating_system
+
+    @property
+    def group(self):
+        """Get host group."""
+        return self._group
+
     @property
     def host_id(self):
         """Get provider host id."""
