@@ -20,14 +20,16 @@ import os
 import socket
 import xml.etree.ElementTree as eTree
 from copy import deepcopy
-from datetime import datetime
+from datetime import datetime, timedelta
 from xml.dom.minidom import Document as xml_doc
+from xmlrpc.client import Fault
 
 from bkr.client import BeakerJob, BeakerRecipe, BeakerRecipeSet
 from bkr.common.hub import HubProxy
 from bkr.common.pyconfig import PyConfigParser
 
-from mrack.errors import ValidationError
+from mrack.context import global_context
+from mrack.errors import ProvisioningError, ValidationError
 from mrack.host import (
     STATUS_ACTIVE,
     STATUS_DELETED,
@@ -36,13 +38,39 @@ from mrack.host import (
     STATUS_PROVISIONING,
 )
 from mrack.providers.provider import STRATEGY_ABORT, Provider
-from mrack.utils import global_context
+from mrack.utils import add_dict_to_node
 
 logger = logging.getLogger(__name__)
 
 PROVISIONER_KEY = "beaker"
 
 
+def parse_bkr_exc_str(exc_str):
+    """Parse exception string and return more readable string for mrack error."""
+    # we expect exception string to look like following:
+    # '<class \'bkr.common.bexceptions.BX\'>:No distro tree matches Recipe:
+    # <distroRequires>
+    #   <and>
+    #     <distro_name op="like" value="Fedora-33%"/>
+    #   </and>
+    # </distroRequires>'
+    if (
+        ":" not in exc_str.faultString
+        and "bkr.common.bexceptions" not in exc_str.faultString
+    ):
+        # we got string we do not expect so just use the traceback
+        return str(exc_str)
+
+    # because of expected format we split by ":" and use last 2 values from list
+    # in above example it would be
+    # [
+    #   '\tNo distro tree matches Recipe',
+    #   '\t<distroRequires><and><distro_name op="like" value="Fedora-33%"/> ...
+    # ]
+    fault = [f"\t{f.strip()}" for f in exc_str.faultString.split(":")[-2:]]
+    return "\n".join(fault)
+
+
 class BeakerProvider(Provider):
     """Beaker Provider."""
 
@@ -50,10 +78,10 @@ class BeakerProvider(Provider):
         """Object initialization."""
         self._name = PROVISIONER_KEY
         self.dsp_name = "Beaker"
-        self.strategy = STRATEGY_ABORT
         self.conf = PyConfigParser()
-        self.poll_sleep = 30  # seconds
+        self.poll_sleep = 45  # seconds
         self.pubkey = None
+        self.max_retry = 1  # for retry strategy
         self.status_map = {
             "Reserved": STATUS_ACTIVE,
             "New": STATUS_PROVISIONING,
@@ -66,14 +94,25 @@ class BeakerProvider(Provider):
             "Cancelled": STATUS_DELETED,
             "Aborted": STATUS_ERROR,
             "Completed": STATUS_OTHER,
+            "MRACK_REACHED_TIMEOUT": STATUS_ERROR,
+            "MRACK_RESULT_NOT_PASSED": STATUS_ERROR,
         }
 
-    async def init(self, distros, max_attempts, reserve_duration, pubkey):
+    async def init(
+        self,
+        distros,
+        timeout,
+        reserve_duration,
+        pubkey,
+        strategy=STRATEGY_ABORT,
+        max_retry=1,
+    ):
         """Initialize provider with data from Beaker configuration."""
         logger.info(f"{self.dsp_name}: Initializing provider")
+        self.strategy = strategy
+        self.max_retry = max_retry
         self.distros = distros
-        # eg: 240 attempts * 30s timeout - 2h timeout for job to complete
-        self.max_attempts = max_attempts
+        self.timeout = timeout
         self.reserve_duration = reserve_duration
         self.pubkey = pubkey
         login_start = datetime.now()
@@ -121,7 +160,7 @@ chmod go-w /root /root/.ssh /root/.ssh/authorized_keys
             % "".join(key_content)
         ]
 
-    def _req_to_bkr_job(self, req):
+    def _req_to_bkr_job(self, req):  # pylint: disable=too-many-locals
         """Transform requirement to beaker job xml."""
         specs = deepcopy(req)  # work with own copy, do not modify the input
 
@@ -156,8 +195,30 @@ chmod go-w /root /root/.ssh /root/.ssh/authorized_keys
         arch_node.setAttribute("value", specs["arch"])
         recipe.addDistroRequires(arch_node)
 
+        host_requires = global_context.PROV_CONFIG[PROVISIONER_KEY].get(
+            "hostRequires",
+            specs.get(f"mrack_{PROVISIONER_KEY}", {}).get("hostRequires", {}),
+        )
+
+        if host_requires:  # suppose to be dict like {"or": [dict()], "and": [dict()]}
+            for operand, operand_value in host_requires.items():
+                if operand.startswith("_"):
+                    recipe.node.getElementsByTagName("hostRequires")[0].setAttribute(
+                        operand[1:],
+                        operand_value,
+                    )
+                    continue
+                # known operands are ["and", "or"]
+                req_node = xml_doc().createElement(operand)
+                for dct in operand_value:
+                    req_node = add_dict_to_node(req_node, dct)
+
+                recipe.node.getElementsByTagName("hostRequires")[0].appendChild(
+                    req_node
+                )
+
         # Specify the custom xml distro_tag node with values from provisioning config
-        distro_tags = global_context["config"]["beaker"].get("distro_tags")
+        distro_tags = global_context.PROV_CONFIG["beaker"].get("distro_tags")
         if distro_tags:
             for tag in distro_tags.get(specs["distro"], []):
                 tag_node = xml_doc().createElement("distro_tag")
@@ -191,30 +252,52 @@ chmod go-w /root /root/.ssh /root/.ssh/authorized_keys
         * 'distro':     beaker distribution to use
         * 'arch':       architecture to request from beaker
         * 'variant':    variant of the system
+
+        Returns:
+            A tuple containing, respectively, a string (<created beaker job id>)
+            and a dict (<requirements for VM>)
+            :rtype: (str, dict)
         """
         logger.info(f"{self.dsp_name}: Creating server")
 
         job = self._req_to_bkr_job(req)  # Generate the job
+        try:
+            job_id = self.hub.jobs.upload(job.toxml())  # schedule beaker job
+        except Fault as bkr_fault:
+            # use the name as id for the logging purposes
+            req["host_id"] = req["name"]
+            raise ProvisioningError(
+                parse_bkr_exc_str(bkr_fault),
+                req,
+            ) from bkr_fault
 
-        job_id = self.hub.jobs.upload(job.toxml())  # schedule beaker job
+        return (job_id, req)
 
-        return (job_id, req["name"])
-
-    def prov_result_to_host_data(self, prov_result):
+    def prov_result_to_host_data(self, prov_result, req):
         """Transform provisioning result to needed host data."""
         try:
             ip_address = socket.gethostbyname(prov_result["system"])
-        except socket.gaierror:
+        except (TypeError, socket.gaierror):
             ip_address = None
 
         result = {
             "id": prov_result["JobID"],
-            "name": prov_result["req_name"],
+            "name": prov_result.get("mrack_req").get("name"),
             "addresses": [ip_address],
             "status": prov_result["status"],
-            "fault": prov_result["result"] if prov_result["result"] != "Pass" else None,
+            "fault": None,
+            "os": prov_result.get("mrack_req").get("os"),
+            "group": prov_result.get("mrack_req").get("group"),
         }
 
+        if prov_result["result"] != "Pass":
+            result.update(
+                {
+                    "fault": prov_result["result"],
+                    "status": "MRACK_RESULT_NOT_PASSED",
+                }
+            )
+
         return result
 
     def _get_recipe_info(self, beaker_id):
@@ -237,22 +320,34 @@ chmod go-w /root /root/.ssh /root/.ssh/authorized_keys
 
     async def wait_till_provisioned(self, resource):
         """Wait for Beaker provisioning result."""
-        beaker_id, req_name = resource
+        beaker_id, req = resource
         resource = {}
-        attempts = 0
         prev_status = ""
+        job_url = ""
 
-        while attempts < self.max_attempts:
-            attempts += 1
+        # let us use timeout variable which is in minutes to define
+        # maximum time to wait for beaker recipe to provide VM
+        timeout_time = datetime.now() + timedelta(minutes=self.timeout)
+
+        while datetime.now() < timeout_time:
             resource = self._get_recipe_info(beaker_id)
             status = resource["status"]
+            job_url = (
+                f"{self.hub._hub_url}"  # pylint: disable=protected-access
+                f"/jobs/{resource['id']}"
+            )
 
             if prev_status != status:
                 logger.info(
-                    f"{self.dsp_name}: Job {beaker_id} has changed "
-                    f"status ({prev_status} -> {status})"
+                    f"{self.dsp_name}: Job {job_url} "
+                    f"has changed status ({prev_status} -> {status})"
                 )
                 prev_status = status
+            else:
+                logger.info(
+                    f"{self.dsp_name}: Job {job_url} has not changed status "
+                    f"({status}), waiting another {self.poll_sleep:.1f}s"
+                )
 
             if self.status_map.get(status) == STATUS_PROVISIONING:
                 await asyncio.sleep(self.poll_sleep)
@@ -260,27 +355,64 @@ chmod go-w /root /root/.ssh /root/.ssh/authorized_keys
                 break
             elif self.status_map.get(status) in [STATUS_ERROR, STATUS_DELETED]:
                 logger.warning(
-                    f"{self.dsp_name}: Job {beaker_id} has errored with status "
+                    f"{self.dsp_name}: Job {job_url} has errored with status "
                     f"{status} and result {resource['result']}"
                 )
+                resource.update({"result": f"Job {job_url} failed to provision"})
                 break
             else:
                 logger.error(
-                    f"{self.dsp_name}: Job {beaker_id} has swithced to unexpected "
+                    f"{self.dsp_name}: Job {job_url} has switched to unexpected "
                     f"status {status} with result {resource['result']}"
                 )
+                resource.update({"result": f"Job {job_url} failed to provision"})
                 break
 
-        resource.update({"JobID": beaker_id, "req_name": req_name})
-        return resource
+        else:
+            # In this case we failed to provision host in time:
+            # we need to create failed host object for mrack
+            # to delete the resource by cancelling the beaker job.
+            logger.error(
+                f"{self.dsp_name}: Job {job_url} failed to provide resource in"
+                f" the timeout of {self.timeout} minutes"
+            )
+            resource.update(
+                {
+                    "status": "MRACK_REACHED_TIMEOUT",
+                    "result": f"Job {job_url} reached timeout",
+                }
+            )
+
+        resource.update(
+            {
+                "JobID": beaker_id,
+                "mrack_req": req,
+            }
+        )
+        return resource, req
 
     async def delete_host(self, host_id):
         """Delete provisioned hosts based on input from provision_hosts."""
-        logger.info(f"{self.dsp_name}: Deleting host by cancelling Job {host_id}")
+        # host_id should start with 'J:' this way we know job has been scheduled
+        # and proper response from beaker hub has beed returned.
+        # Other way (In case of hub error or invalid host definition)
+        # the provider uses hostname from metadata of the VM which has failed
+        # to validate the requirements for the provider
+        if not host_id.startswith("J:"):
+            logger.warning(
+                f"{self.dsp_name}: Job for host '{host_id}' does not exist yet"
+            )
+            return True
+
+        logger.info(
+            f"{self.dsp_name}: Deleting host by cancelling Job "
+            f"{self.hub._hub_url}"  # pylint: disable=protected-access
+            f"/jobs/{host_id.split(':')[1]}"
+        )
         return self.hub.taskactions.stop(
             host_id, "cancel", "Job has been stopped by mrack."
         )
 
-    def to_host(self, provisioning_result, username=None):
+    def to_host(self, provisioning_result, req, username="root"):
         """Transform provisioning result into Host object."""
-        return super().to_host(provisioning_result, username="root")
+        return super().to_host(provisioning_result, req, username)
