@@ -32,9 +32,9 @@ from mrack.errors import (
     ValidationError,
 )
 from mrack.host import STATUS_ACTIVE, STATUS_DELETED, STATUS_ERROR, STATUS_PROVISIONING
-from mrack.providers.provider import STRATEGY_RETRY, Provider
+from mrack.providers.provider import STRATEGY_ABORT, Provider
 from mrack.providers.utils.osapi import ExtraNovaClient, NeutronClient
-from mrack.utils import object2json
+from mrack.utils import get_shortname, is_windows_host, object2json
 
 logger = logging.getLogger(__name__)
 
@@ -61,8 +61,7 @@ class OpenStackProvider(Provider):
         """Object initialization."""
         self._name = PROVISIONER_KEY
         self.dsp_name = "OpenStack"
-        self.strategy = STRATEGY_RETRY
-        self.max_attempts = 5  # provisioning retries
+        self.max_retry = 1  # provisioning retries
         self.flavors = {}
         self.flavors_by_ref = {}
         self.images = {}
@@ -87,7 +86,9 @@ class OpenStackProvider(Provider):
             # https://docs.openstack.org/api-guide/compute/server_concepts.html
         }
 
-    async def init(self, image_names=None, networks=None):
+    async def init(
+        self, image_names=None, networks=None, strategy=STRATEGY_ABORT, max_retry=1
+    ):
         """Initialize provider with data from OpenStack.
 
         Load:
@@ -99,6 +100,8 @@ class OpenStackProvider(Provider):
         """
         # session expects that credentials will be set via env variables
         logger.info(f"{self.dsp_name}: Initializing provider")
+        self.strategy = strategy
+        self.max_retry = max_retry
         try:
             self.session = AuthPassword()
         except TypeError as terr:
@@ -107,6 +110,7 @@ class OpenStackProvider(Provider):
                 "credentials and try again. E.g.: $ source PROJECT-openrc.sh"
             )
             raise NotAuthenticatedError(err) from terr
+
         self.nova = ExtraNovaClient(session=self.session)
         self.glance = GlanceClient(session=self.session)
         self.neutron = NeutronClient(session=self.session)
@@ -122,16 +126,32 @@ class OpenStackProvider(Provider):
 
         self.network_pools = networks
         object_start = datetime.now()
-        _, _, limits, _, _ = await asyncio.gather(
-            self.load_flavors(),
-            self.load_images(image_names),
-            self.nova.limits.show(),
-            self.load_networks(),
-            self.load_ip_availabilities(),
-        )
-        self.limits = limits
-        object_end = datetime.now()
-        object_duration = object_end - object_start
+
+        error_attempts = 0
+        while error_attempts < SERVER_ERROR_RETRY:
+            try:
+                _, _, self.limits, _, _ = await asyncio.gather(
+                    self.load_flavors(),
+                    self.load_images(image_names),
+                    self.nova.limits.show(),
+                    self.load_networks(),
+                    self.load_ip_availabilities(),
+                )
+                break
+            except ServerError as exc:
+                logger.debug(f"{self.dsp_name}: {exc}")
+                error_attempts += 1
+                if error_attempts <= SERVER_ERROR_RETRY:
+                    await asyncio.sleep(SERVER_ERROR_SLEEP)
+                    continue  # Try again due to ServerError
+
+        else:
+            # now we are past to what we would like to wait fail now
+            raise ProvisioningError(
+                f"{self.dsp_name}: Failed to load environment objects from server",
+            )
+
+        object_duration = datetime.now() - object_start
         logger.info(
             f"{self.dsp_name}: Environment objects load duration: {object_duration}"
         )
@@ -186,6 +206,9 @@ class OpenStackProvider(Provider):
         networks = [self.get_network(net) for net in possible_networks]
         usable = []
         for network in networks:
+            if not network:
+                continue
+
             ips = self.get_ips(ref=network.get("id"))
 
             available = ips["total_ips"] - ips["used_ips"]
@@ -281,6 +304,13 @@ class OpenStackProvider(Provider):
         network = self.networks.get(name)
         if not network:
             network = self.networks_by_ref.get(ref)
+
+        if not network:
+            net_id = name or ref
+            logger.debug(
+                f"{self.dsp_name}: Failed to load network with name: '{net_id}'"
+            )
+
         return network
 
     def get_ips(self, name=None, ref=None):
@@ -523,6 +553,12 @@ class OpenStackProvider(Provider):
         name = req.get("name")
         logger.info(f"{self.dsp_name}: Creating server {name}")
         specs = deepcopy(req)  # work with own copy, do not modify the input
+        del specs["os"]  # do not pass this to openstack
+
+        if is_windows_host(req):
+            # Windows support only shortnames for Cloudbase-Init - it is
+            # derived from openstack vm name.
+            specs["name"] = get_shortname(name)
 
         flavor = self._translate_flavor(req)
         specs["flavorRef"] = flavor["id"]
@@ -539,24 +575,20 @@ class OpenStackProvider(Provider):
         if specs.get("network"):
             del specs["network"]
 
+        if specs.get("group"):
+            del specs["group"]
+
         error_attempts = 0
-        while True:
+        while error_attempts < SERVER_ERROR_RETRY:
             try:
                 response = await self.nova.servers.create(server=specs)
             except ServerError as exc:
-                logger.debug(exc)
+                logger.debug(f"{self.dsp_name}: {exc}")
                 error_attempts += 1
                 if error_attempts <= SERVER_ERROR_RETRY:
                     await asyncio.sleep(SERVER_ERROR_SLEEP)
                     continue  # Try again due to ServerError
 
-            if error_attempts > SERVER_ERROR_RETRY:
-                # now we are past to what we would like to wait fail now
-                raise ProvisioningError(
-                    f"{self.dsp_name}: Failed to create server {req['name']}",
-                    req,  # add the requirement dictionary to traceback for later
-                )
-
             fault = response["server"].get("fault", {})
 
             if fault.get("code") == 500:
@@ -579,7 +611,14 @@ class OpenStackProvider(Provider):
                 # provisioning seems to pass correctly break to return result
                 break
 
-        return response.get("server")
+        else:
+            # now we are past to what we would like to wait fail now
+            raise ProvisioningError(
+                f"{self.dsp_name}: Failed to create server {req['name']}",
+                req,  # add the requirement dictionary to traceback for later
+            )
+
+        return (response.get("server"), req)
 
     async def delete_server(self, uuid):
         """Issue deletion of server.
@@ -619,6 +658,7 @@ class OpenStackProvider(Provider):
 
         Return information about provisioned server.
         """
+        resource, req = resource
         uuid = resource.get("id")
 
         poll_sleep_initial = self.poll_sleep_initial + self.poll_init_adj
@@ -626,17 +666,16 @@ class OpenStackProvider(Provider):
             poll_sleep_initial / 2 + poll_sleep_initial * random() * 1.5
         )
         poll_sleep = self.poll_sleep + self.poll_adj
-        timeout = self.timeout
 
         start = datetime.now()
-        timeout_time = start + timedelta(minutes=timeout)
+        timeout_time = start + timedelta(minutes=self.timeout)
 
         # do not check the state immediately, it will take some time
-        logger.debug(f"{uuid}: sleeping for {poll_sleep_initial} seconds")
+        logger.debug(f"{uuid}: sleeping for {poll_sleep_initial:.1f} seconds")
         await asyncio.sleep(poll_sleep_initial)
 
         resp = {}
-        logger.debug(f"Waiting for: {uuid}")
+        logger.debug(f"Waiting for '{req['name']}': {uuid}")
         error_attempts = 0
         while datetime.now() < timeout_time:
             try:
@@ -654,7 +693,7 @@ class OpenStackProvider(Provider):
                 break
 
             poll_sleep += 0.5  # increase delays to check the longer it takes
-            logger.debug(f"{uuid}: sleeping for {poll_sleep} seconds")
+            logger.debug(f"{uuid}: sleeping for {poll_sleep:.1f} seconds")
             await asyncio.sleep(poll_sleep)
 
         done_time = datetime.now()
@@ -663,14 +702,16 @@ class OpenStackProvider(Provider):
         if datetime.now() >= timeout_time:
             logger.warning(
                 f"{self.dsp_name}: Host {uuid} was not provisioned "
-                f"within a timeout of {timeout} mins"
+                f"within a timeout of {self.timeout} mins"
             )
         else:
             logger.info(
                 f"{self.dsp_name}: Host {uuid} was provisioned in {prov_duration:.1f}s"
             )
 
-        return server
+        server.update({"mrack_req": req})
+
+        return server, req
 
     async def delete_host(self, host_id):
         """Issue deletion of host(server) from OpenStack."""
@@ -678,15 +719,17 @@ class OpenStackProvider(Provider):
         await self.delete_server(host_id)
         return True
 
-    def prov_result_to_host_data(self, prov_result):
+    def prov_result_to_host_data(self, prov_result, req):
         """Get needed host information from openstack provisioning result."""
         result = {}
 
         result["id"] = prov_result.get("id")
-        result["name"] = prov_result.get("name")
+        result["name"] = req.get("name")
         networks = prov_result.get("addresses", {})
         result["addresses"] = [ip.get("addr") for n in networks.values() for ip in n]
         result["fault"] = prov_result.get("fault")
         result["status"] = prov_result.get("status")
+        result["os"] = prov_result.get("mrack_req").get("os")
+        result["group"] = prov_result.get("mrack_req").get("group")
 
         return result
