@@ -37,7 +37,7 @@ class PodmanProvider(Provider):
         """Initialize provider."""
         self._name = PROVISIONER_KEY
         self.dsp_name = "Podman"
-        self.strategy = STRATEGY_ABORT
+        self.max_retry = 1  # for retry strategy
         self.podman = Podman()
         self.status_map = {
             STATUS_ACTIVE: STATUS_ACTIVE,
@@ -45,14 +45,26 @@ class PodmanProvider(Provider):
             STATUS_ERROR: STATUS_ERROR,
         }
 
-    async def init(self, container_images, default_network, ssh_key, container_options):
+    async def init(
+        self,
+        container_images,
+        default_network,
+        ssh_key,
+        container_options,
+        extra_commands,
+        strategy=STRATEGY_ABORT,
+        max_retry=1,
+    ):
         """Initialize Podman provider with data from config."""
         logger.info(f"{self.dsp_name}: Initializing provider")
         login_start = datetime.now()
+        self.strategy = strategy
+        self.max_retry = max_retry
         self.images = container_images
         self.default_network = default_network
         self.ssh_key = ssh_key
         self.podman_options = container_options
+        self.extra_commands = extra_commands
         login_end = datetime.now()
         login_duration = login_end - login_start
         logger.info(f"{self.dsp_name}: Init duration {login_duration}")
@@ -103,7 +115,7 @@ class PodmanProvider(Provider):
         if success:
             logger.info(f"{self.dsp_name}: All required images present")
         else:
-            logger.error(f"{self.dsp_name}: Pulling of missing images failed")
+            raise ProvisioningError("Pulling of missing images failed", self.dsp_name)
 
         return success
 
@@ -130,18 +142,23 @@ class PodmanProvider(Provider):
                 "Could not set up podman network for some host(s)", req
             )
 
-        container_id = await self.podman.run(
-            image,
-            hostname,
-            network,
-            extra_options=self.podman_options,
-            remove_at_stop=True,
-        )
-        return container_id
+        try:
+            container_id = await self.podman.run(
+                image,
+                hostname,
+                network,
+                extra_options=self.podman_options,
+                remove_at_stop=True,
+            )
+        except ProvisioningError as p_error:
+            raise ProvisioningError(p_error, req) from p_error
+
+        return (container_id, req)
 
     async def wait_till_provisioned(self, resource):
         """Wait till resource is provisioned."""
-        cont_id = resource
+        # access fist item from tuple resource which should be id
+        cont_id, req = resource
 
         start = datetime.now()
         timeout = 20
@@ -149,14 +166,12 @@ class PodmanProvider(Provider):
 
         while datetime.now() < timeout_time:
             try:
-                inspect = await self.podman.inspect(cont_id)
+                server = await self.podman.inspect(cont_id)[0]
             except ProvisioningError as err:
                 logger.error(f"{self.dsp_name}: {object2json(err)}")
                 raise ServerNotFoundError(cont_id) from err
-            server = inspect[0]
-            running = server["State"]["Running"]
-            is_error = server["State"]["Error"]
-            if running or is_error:
+
+            if server["State"]["Running"] or server["State"]["Error"]:
                 break
             await asyncio.sleep(1)
 
@@ -179,27 +194,40 @@ class PodmanProvider(Provider):
             key_content = key_file.read()
 
         if not await self.podman.exec_command(cont_id, "mkdir -p /root/.ssh/"):
-            raise ProvisioningError(f"Could not copy public key to container {cont_id}")
+            raise ProvisioningError(
+                f"Could not copy public key to container {cont_id}", self.dsp_name
+            )
 
         if not await self.podman.exec_command(
             cont_id, f'echo "{key_content}" >> /root/.ssh/authorized_keys'
         ):
-            raise ProvisioningError(f"Could not copy public key to container {cont_id}")
-
-        if not await self.podman.exec_command(cont_id, "systemctl restart sshd"):
             raise ProvisioningError(
-                f"Failed restarting sshd service in container {cont_id}"
+                f"Could not copy public key to container {cont_id}", self.dsp_name
             )
 
-        return server
+        for command in self.extra_commands:
+            if not await self.podman.exec_command(cont_id, command):
+                raise ProvisioningError(
+                    f"Failed to run '{command}' in container {cont_id}", self.dsp_name
+                )
+
+        server.update({"mrack_req": req})
+
+        return server, req
 
     async def delete_host(self, host_id):
         """Delete provisioned host."""
+        # if there is no container we do nothing
+        if not host_id:
+            logger.debug(f"{self.dsp_name}: Container is not created yet, skipping.")
+            return False
+
         # first we inspect the container to find its networks
         insp_data = await self.podman.inspect(host_id)
-        networks = insp_data[0]["NetworkSettings"]["Networks"]
+        networks = insp_data[0]["NetworkSettings"]["Networks"] if insp_data else []
         # then we destroy the container
-        deleted = await self.podman.rm(host_id, force=True)  # TODO use stop and then rm
+        logger.info(f"{self.dsp_name}: Removing container {host_id}")
+        deleted = await self.podman.rm(host_id, force=True)
         # after that we cleanup the podman network
         for net in networks:
             if await self.podman.network_remove(net):
@@ -220,7 +248,7 @@ class PodmanProvider(Provider):
 
         return status
 
-    def prov_result_to_host_data(self, prov_result):
+    def prov_result_to_host_data(self, prov_result, req):
         """Get needed host information from podman provisioning result."""
         result = {}
 
@@ -248,10 +276,12 @@ class PodmanProvider(Provider):
 
         result["fault"] = error_obj
         result["status"] = status
+        result["os"] = prov_result.get("mrack_req").get("os")
+        result["group"] = prov_result.get("mrack_req").get("group")
 
         return result
 
-    def to_host(self, provisioning_result, username=None):
+    def to_host(self, provisioning_result, req, username=None):
         """Transform provisioning result into Host object."""
         # for containers use always root user
-        return super().to_host(provisioning_result, username="root")
+        return super().to_host(provisioning_result, req, username="root")
