@@ -18,9 +18,10 @@ import logging
 import socket
 from datetime import datetime, timedelta
 
-from mrack.errors import ProvisioningError, ValidationError
+from mrack.context import global_context
+from mrack.errors import ProvisioningError
 from mrack.host import STATUS_ACTIVE, STATUS_OTHER, Host
-from mrack.utils import global_context, ssh_to_host
+from mrack.utils import get_username_pass_and_ssh_key, object2json, ssh_to_host
 
 logger = logging.getLogger(__name__)
 
@@ -28,6 +29,7 @@ STRATEGY_ABORT = "abort"
 STRATEGY_RETRY = "retry"
 RET_CODE = 0  # index to access return code from _wait_for_ssh
 HOST_OBJ = 1  # index to access host object from _wait_for_ssh
+ERROR_OBJ = 0  # default index to access host error which caused ProvisioningError
 SPECS = 1  # default index to access host specs which caused ProvisioningError
 
 
@@ -38,7 +40,7 @@ class Provider:
         """Initialize provider."""
         self._name = "dummy"
         self.dsp_name = "Dummy"
-        self.max_attempts = 1
+        self.max_retry = 1
         self.strategy = STRATEGY_ABORT
         self.status_map = {"OTHER": STATUS_OTHER}
 
@@ -67,27 +69,29 @@ class Provider:
         """Prepare provisioning."""
         raise NotImplementedError()
 
-    async def _wait_for_ssh(self, host, port=22, timeout=1200):
+    async def _wait_for_ssh(self, host, timeout, port):
         """
-        Wait until a port starts accepting TCP connections.
+        Wait until a port starts accepting TCP connections and is able to connect.
 
         Args:
-            port (int): Port number.
             host (Host): Host object to get its address on which the port should exist.
-            timeout (float): In seconds. How long to wait before raising errors.
+            timeout (float): In minutes. How long to wait before raising errors.
+            port (int): Port number.
         Raises:
             TimeoutError: The port isn't accepting connection after specified `timeout`.
         """
         start_time = datetime.now()
         info_msg = (
-            f"{self.dsp_name}: Waiting for the port {port} "
-            f"on host {host.ip_addr} to start accepting connections"
+            f"{self.dsp_name}: Waiting for the port {port} on host "
+            f"{host.ip_addr} to start accepting connections (up to {timeout} minutes)"
         )
         logger.info(info_msg)
 
         while True:
             try:
-                with socket.create_connection((host.ip_addr, port), timeout=timeout):
+                with socket.create_connection(
+                    (host.ip_addr, port), timeout=(timeout * 60)
+                ):
                     logger.info(
                         f"{self.dsp_name}: Port {port} on host "
                         f" {host.ip_addr} is now open"
@@ -96,19 +100,36 @@ class Provider:
             except OSError:
                 await asyncio.sleep(10)
                 logger.debug(info_msg)
-                if datetime.now() - start_time >= timedelta(seconds=timeout):
+                if datetime.now() - start_time >= timedelta(seconds=(timeout * 60)):
                     logger.error(
                         f"{self.dsp_name}: Waited too long for the port {port} "
                         f"on host {host.ip_addr} to start accepting connections"
                     )
-                    break
+                    # do not continue to try ssh connection after port is not open
+                    return False, host
 
         # Wait also for the ssh key to be accepted for a half timeout time
         start_ssh = datetime.now()
+        # load stuff from configs: like:
+        username, password, ssh_key = get_username_pass_and_ssh_key(
+            host, global_context
+        )
+
+        info_msg = (
+            f"{self.dsp_name}: Waiting for the host {host.ip_addr} "
+            f"to accept the authentication method (up to {timeout} minutes)"
+        )
 
         while True:
-            res = ssh_to_host(host, command="echo mrack")
+            res = ssh_to_host(
+                host,
+                username=username,
+                password=password,
+                ssh_key=ssh_key,
+                command="echo mrack",
+            )
             duration = (datetime.now() - start_ssh).total_seconds()
+
             if res:
                 logger.info(
                     f"{self.dsp_name}: SSH to host '{host.ip_addr}' successful "
@@ -116,7 +137,7 @@ class Provider:
                 )
                 break
 
-            if datetime.now() - start_ssh >= timedelta(seconds=(timeout / 2)):
+            if datetime.now() - start_ssh >= timedelta(seconds=(timeout * 60)):
                 logger.error(
                     f"{self.dsp_name}: SSH to host '{host.ip_addr}' "
                     f"timed out after {duration:.1f}s"
@@ -128,9 +149,84 @@ class Provider:
 
         return res, host
 
+    async def _check_ssh_auth(self, default_check, active_hosts):
+        success_hosts = []
+        error_hosts = []
+
+        if not isinstance(default_check, dict):
+            default_check = {}
+
+        # check if default config is in place, if not use defaults
+        req_keys = ("enabled", "port", "timeout")
+        if not all(k in default_check for k in req_keys):
+            logger.warning(
+                f"{self.dsp_name}: Missing complete default post provisioning "
+                "ssh check configuration in provisioning config file."
+            )
+            # extend not complete configuration with defaults
+            default_check = {
+                "enabled": True,
+                "port": 22,
+                "timeout": 10,
+            } | default_check
+
+        # split dictionary into two default and based on host os/group
+        req_keys += ("enabled_providers", "disabled_providers")
+        based_check = {x: default_check[x] for x in default_check if x not in req_keys}
+        default_check = {x: default_check[x] for x in default_check if x in req_keys}
+
+        wait_ssh = []
+        for host in active_hosts:
+            # go host by host and load the group and os based configuration for ssh
+            os_check = based_check.get("os", {}).get(host.operating_system, {})
+            group_check = based_check.get("group", {}).get(host.group, {})
+            opts = default_check | group_check | os_check  # sorted by priority
+
+            logger.debug(
+                f"{self.dsp_name}: Host '{host.name}' "
+                f"ssh check config: {object2json(opts)}"
+            )
+
+            if (
+                not opts.get("enabled")
+                and self.name not in opts.get("enabled_providers", [])
+            ) or (
+                opts.get("enabled") and self.name in opts.get("disabled_providers", [])
+            ):
+                success_hosts.append(host)
+                logger.debug(
+                    f"{self.dsp_name}: Skipping ssh check for host '{host.name}'"
+                )
+                continue
+
+            awaitable = self._wait_for_ssh(
+                host,
+                timeout=opts.get("timeout"),
+                port=opts.get("port"),
+            )
+            wait_ssh.append(awaitable)
+
+        ssh_results = await asyncio.gather(*wait_ssh)
+        # We distinguish the success hosts and new error hosts from active by using:
+        # res[RET_CODE] 0
+        #   - the result of operation returned from self._wait_for_ssh()
+        # res[HOST_OBJ] 1
+        #   - the host object returned from self._wait_for_ssh()
+        for res in ssh_results:
+            if res[RET_CODE]:
+                success_hosts.append(res[HOST_OBJ])
+            else:
+                res[HOST_OBJ].error = (
+                    "Could not establish ssh connection to host "
+                    f"{res[HOST_OBJ].host_id} with IP {res[HOST_OBJ].ip_addr}"
+                )
+                error_hosts.append(res[HOST_OBJ])
+
+        return success_hosts, error_hosts
+
     async def _provision_base(
         self, reqs, res_check_timeout=60, res_busy_sleep=10
-    ):  # pylint: disable=too-many-locals, too-many-branches
+    ):  # pylint: disable=too-many-locals, too-many-branches, too-many-statements
         """Provision hosts based on list of host requirements.
 
         Main function which does provisioning and validation.
@@ -150,13 +246,36 @@ class Provider:
 
         logger.info(f"{self.dsp_name}: Checking available resources")
 
+        error_hosts = []
         res_check_start = datetime.now()
         while not await self.can_provision(reqs):
-            await asyncio.sleep(res_busy_sleep * 60)
             if datetime.now() - res_check_start >= timedelta(minutes=res_check_timeout):
-                raise ValidationError(
-                    f"{self.dsp_name}: Not enough resources to provision"
-                )
+                # create error host object so retry strategy can continue
+                # instead of throwing exception to fail at once without retry
+                err_str = "Not enough resources to provision"
+                logger.error(f"{self.dsp_name}: {err_str}")
+                for req in reqs:
+                    error_hosts.append(
+                        Host(
+                            provider=self,
+                            host_id=req.get("name"),
+                            name=req.get("name"),
+                            operating_system=req.get("os"),
+                            group=req.get("group"),
+                            ip_addrs=[],
+                            status=STATUS_OTHER,
+                            rawdata=req,
+                            error_obj=err_str,
+                        )
+                    )
+                return ([], error_hosts, reqs)
+
+            logger.info(
+                f"{self.dsp_name}: Not enough resources to provision, "
+                f"checking again in {res_busy_sleep} min(s)"
+            )
+            await asyncio.sleep(res_busy_sleep * 60)
+
         logger.info(f"{self.dsp_name}: Resource availability: OK")
         started = datetime.now()
 
@@ -173,27 +292,31 @@ class Provider:
 
         logger.info(f"{self.dsp_name}: Waiting for all hosts to be active")
 
-        error_hosts = []
         wait_servers = []
         for response in create_resps:
-            if not isinstance(response, ProvisioningError):
-                # response might be okay so let us wait for result
-                awaitable = self.wait_till_provisioned(response)
-                wait_servers.append(awaitable)
-            else:
+            if isinstance(response, ProvisioningError):
                 # use ProvisioningError arguments to create missing Host object
                 # which we append to error hosts list for later usage
                 error_hosts.append(
                     Host(
                         provider=self,
-                        host_id=None,
-                        name=response.args[SPECS]["name"],
+                        host_id=response.args[SPECS].get("host_id"),
+                        name=response.args[SPECS].get("name"),
+                        operating_system=response.args[SPECS].get("os"),
+                        group=response.args[SPECS].get("group"),
                         ip_addrs=[],
                         status=STATUS_OTHER,
                         rawdata=response.args,
-                        error_obj=response.args,
+                        error_obj=response.args[ERROR_OBJ],
                     )
                 )
+            elif isinstance(response, Exception):
+                logger.error("An unexpected exception occurred while provisioning")
+                raise response
+            else:
+                # response might be okay so let us wait for result
+                awaitable = self.wait_till_provisioned(response)
+                wait_servers.append(awaitable)
 
         server_results = await asyncio.gather(*wait_servers)
         provisioned = datetime.now()
@@ -204,34 +327,22 @@ class Provider:
         )
         logger.info(f"{self.dsp_name}: Provisioning duration: {provisioned - started}")
 
-        hosts = [self.to_host(srv) for srv in server_results if srv]
+        hosts = [self.to_host(srv, req) for srv, req in server_results if srv]
 
         error_hosts += await self.parse_error_hosts(hosts)
         active_hosts = [h for h in hosts if h not in error_hosts]
         success_hosts = []
 
-        if global_context["config"].get("post_provisioning_ssh_check", True):
-            # check ssh connectivity to succeeded hosts
-            wait_ssh = []
-            for host in active_hosts:
-                awaitable = self._wait_for_ssh(host)
-                wait_ssh.append(awaitable)
+        ssh_check = global_context.PROV_CONFIG.get("post_provisioning_check", {}).get(
+            "ssh", True
+        )  # enable check by default
 
-            ssh_results = await asyncio.gather(*wait_ssh)
-            # We distinguish the success hosts and new error hosts from active by using:
-            # res[RET_CODE] 0
-            #   - the result of operation returned from self._wait_for_ssh()
-            # res[HOST_OBJ] 1
-            #   - the host object returned from self._wait_for_ssh()
-            for res in ssh_results:
-                if res[RET_CODE]:
-                    success_hosts.append(res[HOST_OBJ])
-                else:
-                    res[HOST_OBJ].error = (
-                        "Could not establish ssh connection to host "
-                        f"{res[HOST_OBJ].host_id} with IP {res[HOST_OBJ].ip_addr}"
-                    )
-                    error_hosts.append(res[HOST_OBJ])
+        # check ssh connectivity to hosts if not disabled per host or provider
+        if bool(ssh_check):
+            success_hosts, failed_hosts = await self._check_ssh_auth(
+                ssh_check, active_hosts
+            )
+            error_hosts += failed_hosts
         else:  # we do not check the ssh connection to VMs
             success_hosts = active_hosts
 
@@ -278,8 +389,16 @@ class Provider:
         error_hosts = []
 
         while missing_reqs:
-            if attempts >= self.max_attempts:
-                logger.error(f"Max attempts({self.max_attempts}) reached. Aborting.")
+            # number of attempts should be greater than max_retry
+            # and not greater or equal than max_retry because
+            # other way first attempt to provision will be
+            # considered as first retry and provisioning will end
+            # after reaching count 1 without reprovisioning retry
+            if attempts > self.max_retry:
+                logger.error(
+                    f"{self.dsp_name}: Max retry attempts "
+                    f"({self.max_retry}) reached. Aborting"
+                )
                 break
 
             attempts += 1
@@ -288,14 +407,18 @@ class Provider:
             )
             success_hosts.extend(s_hosts)
 
-            if error_hosts:
+            # We need to cleanup the error hosts for the next retry run
+            # in case of last attempt which is `self.max_retry`
+            # we skip this part at it done in provision_hosts() method while
+            # awaiting the self.abort_and_delete(error hosts)
+            if error_hosts and attempts <= self.max_retry:
                 count = len(error_hosts)
                 err = f"{count} hosts were not provisioned properly, deleting."
                 logger.info(f"{self.dsp_name}: {err}")
                 for host in error_hosts:
                     logger.error(f"{self.dsp_name}: Error: {str(host.error)}")
                 await self.delete_hosts(error_hosts)
-                logger.info("Retrying to provision these hosts.")
+                logger.info(f"{self.dsp_name}: Retrying to provision these hosts.")
 
         return success_hosts, error_hosts, missing_reqs
 
@@ -325,7 +448,11 @@ class Provider:
 
         logger.info(f"{self.dsp_name}: Given the error, will delete hosts")
         await self.delete_hosts(hosts_to_delete)
-        raise ProvisioningError(error_hosts)
+        raise ProvisioningError(
+            f"{self.dsp_name}: Failed to provision {len(error_hosts)} host(s) "
+            "due to provisioning error",
+            self.dsp_name,
+        )
 
     async def delete_host(self, host_id):
         """Delete provisioned host."""
@@ -334,7 +461,6 @@ class Provider:
     async def delete_hosts(self, hosts):
         """Issue deletion of all servers based on previous results from provisioning."""
         logger.info(f"{self.dsp_name}: Issuing deletion")
-
         delete_servers = []
         for host in hosts:
             awaitable = self.delete_host(host.host_id)
@@ -343,18 +469,20 @@ class Provider:
         logger.info(f"{self.dsp_name}: All servers issued to be deleted")
         return results
 
-    def prov_result_to_host_data(self, prov_result):
+    def prov_result_to_host_data(self, prov_result, req):
         """Transform provisioning result to needed host data."""
         raise NotImplementedError()
 
-    def to_host(self, provisioning_result, username=None):
+    def to_host(self, provisioning_result, req, username=None):
         """Transform provisioning result into Host object."""
-        host_info = self.prov_result_to_host_data(provisioning_result)
+        host_info = self.prov_result_to_host_data(provisioning_result, req)
 
         host = Host(
             self,
             host_info.get("id"),
             host_info.get("name"),
+            host_info.get("os"),
+            host_info.get("group"),
             host_info.get("addresses"),
             self.status_map.get(host_info.get("status"), STATUS_OTHER),
             provisioning_result,
