@@ -14,7 +14,8 @@
 
 """AWS transformer module."""
 
-from mrack.transformers.transformer import Transformer
+from mrack.providers.provider import STRATEGY_ABORT
+from mrack.transformers.transformer import DEFAULT_ATTEMPTS, Transformer
 
 CONFIG_KEY = "aws"
 
@@ -41,6 +42,8 @@ class AWSTransformer(Transformer):
             ssh_key=self.config["keypair"],
             sec_group=self.config["security_group"],
             instance_tags=self.config["instance_tags"],
+            strategy=self.config.get("strategy", STRATEGY_ABORT),
+            max_retry=self.config.get("max_retry", DEFAULT_ATTEMPTS),
         )
 
     def create_host_requirement(self, host):
@@ -48,6 +51,8 @@ class AWSTransformer(Transformer):
         required_image = host.get("image") or self._get_image(host["os"])
         return {
             "name": host["name"],
+            "os": host["os"],
+            "group": host["group"],
             "flavor": self._get_flavor(host),
             "image": required_image,
             "meta_image": "image" in host,
