@@ -24,7 +24,8 @@ import logging
 import os
 import subprocess
 import sys
-from typing import Dict
+from functools import update_wrapper
+from xml.dom.minidom import Document as xml_doc
 
 import yaml
 
@@ -32,9 +33,6 @@ from mrack.errors import ConfigError, ProvisioningError
 
 logger = logging.getLogger(__name__)
 
-# Singleton context holder
-global_context: Dict = {}
-
 
 def get_config_value(config_dict, key, default=None):
     """Get configuration value or default value from dictionary.
@@ -67,6 +65,19 @@ def validate_dict_attrs(dct, attr_list, dct_name):
         raise ConfigError(error)
 
 
+def add_dict_to_node(node, input_dict):
+    """Convert dict object to XML elements."""
+    # Recursively create xml from dict
+    if isinstance(input_dict, dict):
+        for key, value in input_dict.items():
+            if key.startswith("_"):
+                node.setAttribute(key[1:], str(value))
+            else:
+                node.appendChild(add_dict_to_node(xml_doc().createElement(key), value))
+
+    return node
+
+
 def json_convertor(obj):
     """Convert object to be useable for json serialization.
 
@@ -176,6 +187,15 @@ def is_windows_host(meta_host):
     )
 
 
+def get_shortname(hostname):
+    """
+    Get shortname part of fqdn.
+
+    Return the same string if it is not fqdn.
+    """
+    return hostname.split(".")[0]
+
+
 def get_username(host, meta_host, config):
     """Find username from sources db/metadata/config."""
     username = host.username or meta_host.get("username")
@@ -208,6 +228,15 @@ def get_ssh_key(host, meta_host, config):
     return ssh_key
 
 
+def get_username_pass_and_ssh_key(host, context):
+    """Return username password and ssh_key to be later used for ssh connections."""
+    meta_host, _domain = get_host_from_metadata(context.METADATA, host.name)
+    username = get_username(host, meta_host, context.PROV_CONFIG)
+    ssh_key = get_ssh_key(host, meta_host, context.PROV_CONFIG)
+    password = None if ssh_key else get_password(host, meta_host, context.PROV_CONFIG)
+    return username, password, ssh_key
+
+
 def ssh_to_host(
     host,
     username=None,
@@ -216,16 +245,13 @@ def ssh_to_host(
     command=None,
 ):
     """SSH to the selected host."""
-    meta_host, _domain = get_host_from_metadata(global_context["metadata"], host.name)
-    username = username or get_username(host, meta_host, global_context["config"])
-    ssh_key = ssh_key or get_ssh_key(host, meta_host, global_context["config"])
     psw = host.password or password
 
-    my_env = os.environ.copy()
-
     run_args = {
-        "env": my_env,
+        "env": os.environ.copy(),
         "shell": True,
+        "stdout": subprocess.PIPE,
+        "stderr": subprocess.PIPE,
     }
 
     cmd = ["ssh"]
@@ -249,9 +275,16 @@ def ssh_to_host(
 
     cmd = " ".join(cmd)
 
-    logger.info(cmd)
+    logger.debug(f"Running: {cmd}")
     process = subprocess.Popen(cmd, **run_args)
-    process.communicate()
+    std_out, std_err = process.communicate()
+
+    for o_line in std_out.decode().splitlines():
+        logger.debug(f"stdout: {o_line}")
+
+    for e_line in std_err.decode().splitlines():
+        logger.debug(f"stderr: {e_line}")
+
     return process.returncode == 0
 
 
@@ -277,6 +310,17 @@ async def exec_async_subprocess(program, args, raise_on_err=True):
     return stdout, stderr, process
 
 
+def async_run(func):
+    """Decorate click actions to run as async."""
+    func = asyncio.coroutine(func)
+
+    def wrapper(*args, **kwargs):
+        loop = asyncio.get_event_loop()
+        return loop.run_until_complete(func(*args, **kwargs))
+
+    return update_wrapper(wrapper, func)
+
+
 class NoSuchFileHandler:
     """
     Decorator which change error into ConfigError with custom message.
